@@ -28,12 +28,14 @@ class Alarmdotcom:
     SYSTEMITEMS_URL = "https://www.alarm.com/web/api/systems/availableSystemItems"
     SYSTEM_URL_TEMPLATE = "{}web/api/systems/systems/{}"
     PARTITION_URL_TEMPLATE = "{}web/api/devices/partitions/{}"
+    LOCK_URL_TEMPLATE = "{}web/api/devices/locks/{}"
     TROUBLECONDITIONS_URL_TEMPLATE = (
         "{}web/api/troubleConditions/troubleConditions?forceRefresh=false"
     )
     SENSOR_STATUS_URL_TEMPLATE = "{}web/api/devices/sensors"
     THERMOSTAT_STATUS_URL_TEMPLATE = "{}web/api/devices/thermostats"
     GARAGE_DOOR_STATUS_URL_TEMPLATE = "{}web/api/devices/garageDoors"
+    LOCK_STATUS_URL_TEMPLATE = "{}web/api/devices/locks"
     STATEMAP = (
         "",
         "disarmed",
@@ -46,10 +48,17 @@ class Alarmdotcom:
         "Open",  # 1
         "Closed",  # 2
     )
+    LOCK_STATEMAP = (
+        "Failed",
+        "Locked",
+        "Open",
+    )
     COMMAND_LIST = {
         "Disarm": {"command": "disarm"},
         "Arm+Stay": {"command": "armStay"},
         "Arm+Away": {"command": "armAway"},
+        "Lock": {"command": "lock"},
+        "Unlock": {"command": "unlock"},
     }
     THERMOSTAT_ATTRIBUTES = (
         "ambientTemp",
@@ -83,16 +92,19 @@ class Alarmdotcom:
         self._websession = websession
         self.state = ""  # empty string instead of None
         self.sensor_status = None
+        self.sensor_status_detailed = dict()
         self._ajax_headers = {
             "Accept": "application/vnd.api+json",
             "ajaxrequestuniquekey": None,
         }
-        self._systemid = None
+        self.systemid = None
         self._partitionid = None
+        self._lockid = None
         self._forcebypass = forcebypass  # "stay","away","true","false"
         self._noentrydelay = noentrydelay  # "stay","away","true","false"
         self._silentarming = silentarming  # "stay","away","true","false"
         self._thermostat_detected = False
+        self._lock_detected = False
         self._garage_door_detected = False
         self._url_base = self.URL_BASE
         self._twofactor_cookie = (
@@ -163,7 +175,7 @@ class Alarmdotcom:
                 url=self.SYSTEMITEMS_URL, headers=self._ajax_headers
             ) as resp:
                 json = await (resp.json())
-            self._systemid = json["data"][0]["id"]
+            self.systemid = json["data"][0]["id"]
         except (asyncio.TimeoutError, aiohttp.ClientError):
             _LOGGER.error("Can not load system data from Alarm.com")
             return False
@@ -171,15 +183,22 @@ class Alarmdotcom:
             _LOGGER.error("Unable to extract system id from Alarm.com")
             raise
         try:
-            # grab partition id
+            # grab partition id and lock id
             async with self._websession.get(
-                url=self.SYSTEM_URL_TEMPLATE.format(self._url_base, self._systemid),
+                url=self.SYSTEM_URL_TEMPLATE.format(self._url_base, self.systemid),
                 headers=self._ajax_headers,
             ) as resp:
                 json = await (resp.json())
             self._partitionid = json["data"]["relationships"]["partitions"]["data"][0][
                 "id"
             ]
+
+            locks = json["data"]["relationships"].get("locks", {}).get("data", [])
+
+            self._lock_detected = len(locks) > 0
+            if self._lock_detected:
+                self._lockid = json["data"]["relationships"]["locks"]["data"][0]["id"]
+
             thermostats = (
                 json["data"]["relationships"].get("thermostats", {}).get("data", [])
             )
@@ -205,31 +224,44 @@ class Alarmdotcom:
             return False
         return await self._async_get_system_info()
 
-    async def async_update(self):
-        """Fetch the latest state."""
+    async def async_update(self, mode="alarm"):
+        """Fetch the latest state according to mode."""
         _LOGGER.debug("Calling update on Alarm.com")
         if not self._ajax_headers["ajaxrequestuniquekey"]:
             await self.async_login()
         try:
-            # grab partition status
-            async with self._websession.get(
-                url=self.PARTITION_URL_TEMPLATE.format(
+            if mode == "lock":
+                state_url = self.LOCK_URL_TEMPLATE.format(self._url_base, self._lockid)
+            else:
+                state_url = self.PARTITION_URL_TEMPLATE.format(
                     self._url_base, self._partitionid
-                ),
+                )
+
+            # grab status
+            async with self._websession.get(
+                url=state_url,
                 headers=self._ajax_headers,
             ) as resp:
                 json = await (resp.json())
-            self.sensor_status = json["data"]["attributes"]["needsClearIssuesPrompt"]
-            self.sensor_status = (
-                "System needs to be cleared" if self.sensor_status else "System OK"
-            )
             self.state = json["data"]["attributes"]["state"]
-            self.state = self.STATEMAP[self.state]
+            self.sensor_status = ""
+            if mode == "lock":
+                self.state = self.LOCK_STATEMAP[self.state]
+            else:
+                self.state = self.STATEMAP[self.state]
+                self.sensor_status = json["data"]["attributes"][
+                    "needsClearIssuesPrompt"
+                ]
+                self.sensor_status = (
+                    "System needs to be cleared" if self.sensor_status else "System OK"
+                )
+
             _LOGGER.debug(
                 "Got state %s, mapping to %s",
                 json["data"]["attributes"]["state"],
                 self.state,
             )
+
         except (asyncio.TimeoutError, aiohttp.ClientError):
             _LOGGER.error("Can not load state data from Alarm.com")
             return False
@@ -239,7 +271,8 @@ class Alarmdotcom:
             self.state = None
             self.sensor_status = None
             self._ajax_headers["ajaxrequestuniquekey"] = None
-            await self.async_update()
+            await self.async_update(mode)
+
         try:
             async with self._websession.get(
                 url=self.SENSOR_STATUS_URL_TEMPLATE.format(self._url_base),
@@ -253,12 +286,43 @@ class Alarmdotcom:
                     + " is "
                     + sensor["attributes"]["displayStateText"]
                 )
+
+                # Populate detailed sensor data
+                self.sensor_status_detailed[sensor["id"]] = {
+                    "deviceType": sensor["attributes"].get("deviceType"),
+                    "isMalfunctioning": sensor["attributes"].get("isMalfunctioning"),
+                    "lowBattery": sensor["attributes"].get("lowBattery"),
+                    "criticalBattery": sensor["attributes"].get("criticalBattery"),
+                    "description": sensor["attributes"].get("description"),
+                    "openClosedStatus": sensor["attributes"].get("openClosedStatus"),
+                }
         except (asyncio.TimeoutError, aiohttp.ClientError):
             _LOGGER.error("Can not load sensor status from Alarm.com")
             return False
         except KeyError:
             _LOGGER.error("Unable to extract sensor status from Alarm.com")
             raise
+
+        if self._lock_detected:
+            try:
+                async with self._websession.get(
+                    url=self.LOCK_STATUS_URL_TEMPLATE.format(self._url_base),
+                    headers=self._ajax_headers,
+                ) as resp:
+                    json = await (resp.json())
+                for sensor in json["data"]:
+                    lock_state = sensor["attributes"]["state"]
+                    lock_state = self.LOCK_STATEMAP[lock_state]
+                    self.sensor_status += (
+                        ", " + sensor["attributes"]["description"] + " is " + lock_state
+                    )
+            except (asyncio.TimeoutError, aiohttp.ClientError):
+                _LOGGER.error("Can not load lock status from Alarm.com")
+                return False
+            except KeyError:
+                _LOGGER.error("Unable to extract lock status from Alarm.com")
+                raise
+
         if self._thermostat_detected:
             try:
                 async with self._websession.get(
@@ -326,10 +390,12 @@ class Alarmdotcom:
             raise
         return True
 
-    async def _send(self, event, forcebypass, noentrydelay, silentarming):
+    async def _send(self, mode, event, forcebypass, noentrydelay, silentarming):
         """Generic function for sending commands to Alarm.com
 
         :param event: Event command to send to alarm.com
+
+        :param mode: lock or partition
         """
         _LOGGER.debug("Sending %s to Alarm.com", event)
         if event == "Disarm":
@@ -347,17 +413,26 @@ class Alarmdotcom:
                     if value is True
                 },
             }
+        if mode == "alarm":
+            url_prefix = self.PARTITION_URL_TEMPLATE.format(
+                self._url_base, self._partitionid
+            )
+            _LOGGER.debug("Url prefix %s", url_prefix)
+        elif mode == "lock":
+            url_prefix = self.LOCK_URL_TEMPLATE.format(self._url_base, self._lockid)
+            _LOGGER.debug("Url prefix %s", url_prefix)
+        else:
+            _LOGGER.debug("No such mode %s", mode)
+            return False
         async with self._websession.post(
-            url=self.PARTITION_URL_TEMPLATE.format(self._url_base, self._partitionid)
-            + "/"
-            + self.COMMAND_LIST[event]["command"],
+            url=url_prefix + "/" + self.COMMAND_LIST[event]["command"],
             json=json,
             headers=self._ajax_headers,
         ) as resp:
             _LOGGER.debug("Response from Alarm.com %s", resp.status)
             if resp.status == 200:
                 # Update alarm.com status after calling state change.
-                await self.async_update()
+                await self.async_update(mode)
             if resp.status == 403:
                 # May have been logged out, try again
                 _LOGGER.warning(
@@ -370,6 +445,10 @@ class Alarmdotcom:
                     await self.async_alarm_arm_stay()
                 elif event == "Arm+Away":
                     await self.async_alarm_arm_away()
+                elif event == "Lock":
+                    await self.async_lock()
+                elif event == "Unlock":
+                    await self.async_unlock()
             elif resp.status >= 400:
                 _LOGGER.error("%s failed with HTTP code %s", event, resp.status)
                 _LOGGER.error(
@@ -382,21 +461,29 @@ class Alarmdotcom:
 
     async def async_alarm_disarm(self):
         """Send disarm command."""
-        await self._send("Disarm", False, False, False)
+        await self._send("alarm", "Disarm", False, False, False)
 
     async def async_alarm_arm_stay(self):
         """Send arm stay command."""
         forcebypass = self._forcebypass in ["stay", "true"]
         noentrydelay = self._noentrydelay in ["stay", "true"]
         silentarming = self._silentarming in ["stay", "true"]
-        await self._send("Arm+Stay", forcebypass, noentrydelay, silentarming)
+        await self._send("alarm", "Arm+Stay", forcebypass, noentrydelay, silentarming)
 
     async def async_alarm_arm_away(self):
         """Send arm away command."""
         forcebypass = self._forcebypass in ["away", "true"]
         noentrydelay = self._noentrydelay in ["away", "true"]
         silentarming = self._silentarming in ["away", "true"]
-        await self._send("Arm+Away", forcebypass, noentrydelay, silentarming)
+        await self._send("alarm", "Arm+Away", forcebypass, noentrydelay, silentarming)
+
+    async def async_lock(self):
+        """Send lock command."""
+        await self._send("lock", "Lock", False, False, False)
+
+    async def async_unlock(self):
+        """Send lock command."""
+        await self._send("lock", "Unlock", False, False, False)
 
 
 class AlarmdotcomADT(Alarmdotcom):
@@ -446,7 +533,7 @@ class AlarmdotcomADT(Alarmdotcom):
             ) as resp:
                 json = await resp.json()
             adt_id = json["value"]["id"]
-            self._systemid = json["value"]["customerId"]
+            self.systemid = json["value"]["customerId"]
             await self._websession.post(
                 url=self.SKIP_2FA_URL_TEMPLATE.format(self._url_base, adt_id),
                 headers=self._ajax_headers,
@@ -465,13 +552,14 @@ class AlarmdotcomADT(Alarmdotcom):
                 self._ajax_headers["ajaxrequestuniquekey"] = resp.cookies["afg"].value
             # grab partition id
             async with self._websession.get(
-                url=self.SYSTEM_URL_TEMPLATE.format(self._url_base, self._systemid),
+                url=self.SYSTEM_URL_TEMPLATE.format(self._url_base, self.systemid),
                 headers=self._ajax_headers,
             ) as resp:
                 json = await (resp.json())
             self._partitionid = json["data"]["relationships"]["partitions"]["data"][0][
                 "id"
             ]
+            self._lockid = json["data"]["relationships"]["locks"]["data"][0]["id"]
             thermostats = (
                 json["data"]["relationships"].get("thermostats", {}).get("data", [])
             )
